"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseResource = exports.getSessionKeyUri = exports.getSessionSigs = void 0;
const constants_1 = require("@litprotocol-dev/constants");
const lit_siwe_1 = require("lit-siwe");
const tweetnacl_1 = require("tweetnacl");
const uint8arrays_1 = require("@litprotocol-dev/uint8arrays");
const misc_browser_1 = require("@litprotocol-dev/misc-browser");
const crypto_1 = require("@litprotocol-dev/crypto");
const auth_browser_1 = require("./auth-browser");
/** ========== Local Helpers ========== */
/**
 * Get the session key pair from local storage if it exists,
 * otherwise generate a new one
 *
 * @returns { SessionKeyPair } sessionKey
 */
const getSessionKey = () => {
    const storageKey = constants_1.LOCAL_STORAGE_KEYS.SESSION_KEY;
    // check if we already have a session key + signature for this chain
    let sessionKeyOrError = (0, misc_browser_1.getStorageItem)(storageKey);
    let sessionKey;
    // if we don't have a session key, generate one
    if (sessionKeyOrError.type === "ERROR" /* EITHER_TYPE.ERROR */) {
        sessionKey = (0, crypto_1.generateSessionKeyPair)();
        localStorage.setItem(storageKey, JSON.stringify(sessionKey));
    }
    else {
        sessionKey = JSON.parse(sessionKeyOrError.result);
    }
    return sessionKey;
};
/**
 * Get the wallet signature from local storage if it exists,
 * otherwise check and sign auth message
 *
 * @returns { JsonAuthSig } walletSignature
 */
const getWalletSignature = async (params, sessionKeyUri) => {
    const storageKey = constants_1.LOCAL_STORAGE_KEYS.WALLET_SIGNATURE;
    // check if we already have a wallet signature
    let walletSignatureOrError = (0, misc_browser_1.getStorageItem)(storageKey);
    let walletSignature;
    // if we don't have a wallet signature
    if (walletSignatureOrError.type === "ERROR" /* EITHER_TYPE.ERROR */) {
        walletSignature = await (0, auth_browser_1.checkAndSignAuthMessage)({
            chain: params.chain,
            resources: params.sessionCapabilities,
            switchChain: params.switchChain,
            expiration: params.expiration,
            uri: sessionKeyUri,
        });
        localStorage.setItem(storageKey, JSON.stringify(walletSignature));
    }
    else {
        walletSignature = JSON.parse(walletSignatureOrError.result);
    }
    return walletSignature;
};
/** ========== Exports ========== */
/**
 *
 * High level, how this works:
 * 1. Generate or retrieve session key
 * 2. Generate or retrieve the wallet signature of the session key
 * 3. Sign the specific resources with the session key
 *
 * @param { SessionSigsProp } params
 *
 * @returns
 */
async function getSessionSigs(params) {
    // ========== Prepare Params ==========
    let { expiration, chain, resources = [], sessionCapabilities, switchChain, litNodeClient, } = params;
    // -- get session key
    const sessionKey = getSessionKey();
    // -- get session key URI
    let sessionKeyUri = (0, exports.getSessionKeyUri)({ publicKey: sessionKey.publicKey });
    // -- get sessionCapabilities
    // if the user passed no sessionCapabilities, let's create them for them
    // with wildcards so the user doesn't have to sign every time
    if (!sessionCapabilities || sessionCapabilities.length === 0) {
        sessionCapabilities = resources.map((resource) => {
            const { protocol, resourceId } = (0, exports.parseResource)({ resource });
            return `${protocol}Capability://*`;
        });
    }
    // -- get wallet signature
    let walletSig = await getWalletSignature(params, sessionKeyUri);
    //   ========== Validate Signature ==========
    // Check a few things, including that:
    // 1. the sig isn't expired
    // 2. the sig is for the correct session key
    // 3. the sig has the sessionCapabilities requires to fulfill the resources requested
    // NOTE: "verify" doesn't exist on SwieMessage
    const siweMessage = new lit_siwe_1.SiweMessage(walletSig.signedMessage);
    let needToReSignSessionKey = false;
    try {
        // make sure it's legit
        await siweMessage.verify({ signature: walletSig.sig });
    }
    catch (e) {
        needToReSignSessionKey = true;
    }
    // make sure the sig is for the correct session key
    if (siweMessage.uri !== sessionKeyUri) {
        needToReSignSessionKey = true;
    }
    // make sure the sig has the session capabilities required to fulfill the resources requested
    for (let i = 0; i < resources.length; i++) {
        const resource = resources[i];
        const { protocol, resourceId } = (0, exports.parseResource)({ resource });
        // check if we have blanket permissions or if we authed the specific resource for the protocol
        const permissionsFound = sessionCapabilities.some((capability) => {
            const capabilityParts = (0, exports.parseResource)({ resource: capability });
            return (capabilityParts.protocol === protocol &&
                (capabilityParts.resourceId === '*' ||
                    capabilityParts.resourceId === resourceId));
        });
        if (!permissionsFound) {
            needToReSignSessionKey = true;
        }
    }
    if (needToReSignSessionKey) {
        walletSig = await (0, auth_browser_1.checkAndSignAuthMessage)({
            chain,
            resources: sessionCapabilities,
            switchChain,
            expiration,
            uri: sessionKeyUri,
        });
    }
    //   ========== Sign Resources with Session Key ==========
    // okay great, now we have a valid signed session key
    // let's sign the resources with the session key
    // 5 minutes is the default expiration for a session signature
    // because we can generate a new session sig every time the user wants to access a resource
    // without prompting them to sign with their wallet
    let sessionExpiration = new Date(Date.now() + 1000 * 60 * 5);
    const signingTemplate = {
        sessionKey: sessionKey.publicKey,
        resources,
        capabilities: [walletSig],
        issuedAt: new Date().toISOString(),
        expiration: sessionExpiration.toISOString(),
    };
    const signatures = {};
    litNodeClient.connectedNodes.forEach((nodeAddress) => {
        const toSign = {
            ...signingTemplate,
            nodeAddress,
        };
        let signedMessage = JSON.stringify(toSign);
        const uint8arrayKey = (0, uint8arrays_1.uint8arrayFromString)(sessionKey.secretKey, 'base16');
        const uint8arrayMessage = (0, uint8arrays_1.uint8arrayFromString)(signedMessage, 'utf8');
        let signature = tweetnacl_1.default.sign.detached(uint8arrayMessage, uint8arrayKey);
        // console.log("signature", signature);
        signatures[nodeAddress] = {
            sig: (0, uint8arrays_1.uint8arrayToString)(signature, 'base16'),
            derivedVia: 'litSessionSignViaNacl',
            signedMessage,
            address: sessionKey.publicKey,
            algo: 'ed25519',
        };
    });
    return signatures;
}
exports.getSessionSigs = getSessionSigs;
/**
 *
 * Get Session Key URI eg. lit:session:
 *
 * @returns { string }
 *
 */
const getSessionKeyUri = ({ publicKey, }) => {
    return constants_1.LIT_SESSION_KEY_URI + publicKey;
};
exports.getSessionKeyUri = getSessionKeyUri;
/**
 *
 * Parse resource
 *
 * @property { any } resource
 *
 * @returns { { protocol: string, resourceId: string } }
 *
 */
const parseResource = ({ resource, }) => {
    const [protocol, resourceId] = resource.split('://');
    return { protocol, resourceId };
};
exports.parseResource = parseResource;
//# sourceMappingURL=session.js.map